import { Middleware } from './middleware';
import { RouteBuilder } from './route-builder';
/**
 * This class is in charge of coordinating events from an event source
 * and turning them into actions to be executed based on routes.
 *
 * @export
 * @class Eventline
 */
export declare class Eventline {
    /**
     * Function for handling what happens when an exception occurs.
     *
     * @memberof Eventline
     */
    exceptionHandler: (exception: any, event: any) => void;
    /**
     * This subject is used internally to initiate the
     * execution of middleware used before a route's actions
     *
     * @private
     * @type {Rx.Subject<object>}
     * @memberof Router
     */
    private beforeMiddlewareSubject;
    /**
     * This subject is used internally to initiate the
     * execution of middleware used after a route's actions
     *
     * @private
     * @type {Rx.Subject<object>}
     * @memberof Router
     */
    private afterMiddlewareSubject;
    /**
     * An internal array used to hold all of the
     * registered middleware for this router
     *
     * @private
     * @type {Array<Middleware>}
     * @memberof Router
     */
    private middlewares;
    /**
     * An internal array used to hold all of the
     * registered routes for this router
     *
     * @private
     * @type {Array<Route>}
     * @memberof Router
     */
    private routes;
    /**
     * Passes an event for the router to route
     *
     * @param {object} event
     * @memberof Router
     */
    route(event: object): void;
    /**
     * Passes a component for the router to consume
     *
     * @param {(Router) => void} component
     * @memberof Router
     */
    use(component: (Router) => void): void;
    /**
     * Registers middleware with
     * the router
     *
     * @param {Middleware} middleware
     * @memberof Router
     */
    registerMiddleware(middleware: Middleware): void;
    /**
     * Registers routes for the router
     *
     * @param {*} patterns
     * @returns RouteBuilder
     * @memberof Router
     */
    on(...patterns: any[]): RouteBuilder;
    /**
     * Initiates the router and builds the routes
     * for the runtime
     *
     * @memberof Router
     */
    start(): void;
    /**
     * An internal method for bulding a middleware execution
     * flow for the runtime
     *
     * @param {string} type
     * @param {Rx.Subject<any>} subject
     * @returns {Rx.Observable<any>}
     * @memberof Router
     */
    private buildMiddlewareChain(type, subject);
    /**
     * An internal method for bulding a route execution
     * flow for the runtime
     *
     * @param {Route} route
     * @memberof Router
     */
    private listenToRoute(route);
    /**
     * An internal method for taking an event
     * finding a matching router and triggering it.
     *
     * @param {*} event
     * @memberof Router
     */
    private handleEvent(event);
    /**
     * An internal method which finds
     * a matching route given an event
     *
     * Each route is evaluated in the
     * order it was declared.
     *
     * @param {*} event
     * @returns
     * @memberof Router
     */
    private routeForEvent(event);
}
